#include "../../arduino.c"
#include "stdlib.h"

#define DEBUG 0

#define MIN_OUTPUT 64
#define MAX_OUTPUT 255

// how fast do we want to change (ms)
#define MIN_CHANGE 100
#define MAX_CHANGE 5000

// how fast do we want to fade (ms) at a difference of up to a quarter output
#define MIN_FADE 100
#define MAX_FADE 500
// the factor to apply to 1/64th of the output difference over a quarter
#define MIN_FACTOR 450
#define MAX_FACTOR 750

static int pins[6] = {3, 5, 6, 9, 10, 11};

static uint32_t next_change[6] = {0, 0, 0, 0, 0, 0};
static uint32_t fading_since[6] = {0, 0, 0, 0, 0, 0};
static uint16_t fading_time[6] = {0, 0, 0, 0, 0, 0};
static uint8_t fading_from[6] = {255, 255, 255, 255, 255, 255};
static uint8_t fading_to[6] = {255, 255, 255, 255, 255, 255};


void setup() {
    for (uint8_t i = 0; i < 6; i++) {
        pinMode(pins[i], OUTPUT);
        analogWrite(pins[i], fading_to[i]);
    }
    if (DEBUG) {
        Serial.begin(115200);
    }
}

void loop() {
    uint32_t now = millis();
    for (uint8_t i = 0; i < 6; i++) {
        if (fading_since[i]) {
            if (now > fading_since[i] + fading_time[i]) {
                fading_since[i] = 0;
                analogWrite(pins[i], fading_to[i]);
                // schedule the next change
                uint32_t wait = random(MAX_CHANGE - MIN_CHANGE) + MIN_CHANGE;
                next_change[i] = now + wait;
            } else {
                uint8_t output = (fading_from[i] +
                        ((now - fading_since[i]) / (float)fading_time[i]) *
                        (fading_to[i] - fading_from[i]));
                analogWrite(pins[i], output);
            }
        }
        else if (next_change[i] < now) {
            // schedule the fade
            fading_since[i] = now;
            fading_from[i] = fading_to[i];
            fading_to[i] = random(MAX_OUTPUT - MIN_OUTPUT) + MIN_OUTPUT;

            int16_t diff;
            if (fading_from[i] > fading_to[i]) {
                diff = fading_from[i] - fading_to[i];
            } else {
                diff = fading_to[i] - fading_from[i];
            }
            float modifier = (float)max(0, diff - 64) / 64;
            uint16_t min_fade = MIN_FADE + (modifier * MIN_FACTOR);
            uint16_t max_fade = MAX_FADE + (modifier * MAX_FACTOR);
            fading_time[i] = random(max_fade - min_fade) + min_fade;
            if (DEBUG) {
                Serial.print("modifier: ");
                Serial.println(modifier);
                Serial.print("min_fade: ");
                Serial.println(min_fade);
                Serial.print("max_fade: ");
                Serial.println(max_fade);

                Serial.print(fading_from[i]);
                Serial.print(" -> ");
                Serial.print(fading_to[i]);
                Serial.print(" (");
                Serial.print(diff);
                Serial.print(" ) in ");
                Serial.println(fading_time[i]);

                Serial.println("");
            }
        }
    }
}
